/*
 * Copyright (c) 2010 David McIntosh <dmcintosh@df12.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <math.h>
#include <limits.h>
//#include <float.h>
#import <AppKit/AppKit.h>
#import "util.h"
#import "constants.h"
#import "GanttCell.h"
#import "GanttTask.h"
#import "GanttRulerView.h"
#import "NSDate+GanttAdditions.h"

@interface GanttView : NSControl
{
    GanttCell *emptyCell;           
    NSMutableArray *cells;          // Cells to be displayed
    id delegate;
    NSDate *originDate;             // the date at (0, 0)
    GanttZoomFactor _zoomFactor;
    NSTableView *_tableView;
    NSMutableSet *selection;      // An array of selected cells
    NSMutableSet *expandedTags;
    BOOL sendSelectionDelegateMessages;
}

- (void)awakeFromGSMarkup;
- (void)setDelegate:(id)obj;
- (void)setTableView:(NSTableView *)tableView;
- (void)setOriginDate:(NSDate *)aDate;
- (NSDate *)originDate;
- (void)collapseAll:(id)sender;
- (void)setZoomFactor:(GanttZoomFactor)aFactor;
- (GanttZoomFactor)zoomFactor;
- (NSTimeInterval)extentInterval;
- (void)clearSelection;
- (void)selectTag:(NSInteger)tag;
- (void)deselectTag:(NSInteger)tag;
- (NSSet *)selectedTags;
- (void)selectRowIndexes:(NSIndexSet *)indexes byExtendingSelection:(BOOL)extend;
- (NSIndexSet *)selectedRowIndexes;


- (GanttTask *)taskAtIndex:(NSInteger)index withRoot:(GanttTask *)root;
- (GanttTask *)taskAtIndex:(NSInteger)index;
- (NSInteger)visibleTasksWithRoot:(GanttTask *)root;
- (NSInteger)visibleTasks;

- (void)windowDidResize:(NSNotification *)note;
- (void)taskDidExpand:(NSNotification *)note;
- (void)taskDidCollapse:(NSNotification *)note;

@end
